package com.fossgalaxy.games.tbs.editor.EditorView;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.entity.HexagonTile;
import com.fossgalaxy.games.tbs.entity.Resource;
import com.fossgalaxy.games.tbs.entity.SpriteDef;
import com.fossgalaxy.games.tbs.io.SpriteRegistry;
import com.fossgalaxy.games.tbs.io.map2.EntityDef;
import com.fossgalaxy.games.tbs.io.map2.MapDef;
import com.fossgalaxy.games.tbs.io.map2.ResourceDef;
import com.fossgalaxy.games.tbs.parameters.EntityType;
import com.fossgalaxy.games.tbs.parameters.GameSettings;
import com.fossgalaxy.games.tbs.parameters.ResourceType;
import com.fossgalaxy.games.tbs.parameters.TerrainType;
import com.fossgalaxy.games.tbs.ui.GameAction;
import com.fossgalaxy.games.tbs.ui.GameView;
import com.fossgalaxy.games.tbs.ui.UIModelListener;
import org.codetome.hexameter.core.api.CubeCoordinate;
import org.codetome.hexameter.core.api.Hexagon;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

import static com.fossgalaxy.games.tbs.ui.GameView.hex2shape;

/**
 * Created by webpigeon on 13/10/17.
 */
public class EditorView extends JComponent {

    private EditorModel model;
    private SpriteRegistry sprites;

    public EditorView(EditorModel model) {
        this.sprites = SpriteRegistry.INSTANCE;
        this.setPreferredSize(new Dimension(800, 600));
        this.model = model;

        model.addEditorListener(new EditorListener() {
            @Override
            public void onMapDefLoaded(MapDef def) {

                GameState state = def.buildState(model.getSettings());
                setPreferredSize(new Dimension(state.getGridWidthPixels(), state.getGridHeightPixels()));

                repaint();
            }

            @Override
            public void onEntitySelected(EntityDef def) {
                repaint();
            }

            @Override
            public void onLocationSelected(CubeCoordinate pos) {
                repaint();
            }

        });
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        MapDef def = model.getDef();
        if (def == null) {
            return;
        }

        GameState state = def.buildState(model.getSettings());

        Graphics2D g2 = (Graphics2D) g;

        // Background
        g2.setBackground(Color.BLACK);
        g2.clearRect(0, 0, getWidth(), getHeight());

        // Hexagons
        state.forAllHexagons((Hexagon<HexagonTile> hexagon) -> {
            Shape s = hex2shape.apply(hexagon);

            //draw the terrain type
            TerrainType type = hexagon.getSatelliteData().get().getTerrain();
            if (type != null) {
                BufferedImage terrainImg = sprites.getImage(type.getImage());
                Rectangle bounds = s.getBounds();
                g2.drawImage(terrainImg, (int) bounds.getX(), (int) bounds.getY(), (int) bounds.width + 1, (int) bounds.height + 1, null);
            }

            //draw the hex outline
            g2.setColor(Color.DARK_GRAY);
            g2.draw(s);

            //if we're showing locations, draw the locations.
            if (model.isShowingLocations()) {
                g2.setColor(Color.WHITE);
                String location = String.format("%dx %dy %dz", hexagon.getGridX(), hexagon.getGridY(), hexagon.getGridZ());
                g2.drawString(location, (int) hexagon.getCenterX() - 30, (int) hexagon.getCenterY());
            }
        });

        // paint things
        paintResources(g2, def, state);
        paintEntities(g2, def, state);

        //highlight the currently selected grid cell
        CubeCoordinate target = model.getSelected();
        if (target != null) {
            Shape hexShape = state.computeHexagonAt(target, hex2shape);
            g.setColor(Color.RED);
            g2.draw(hexShape);
        }

        //highlight the currently selected entity
        EntityDef highlight = model.getEntityDef();

        if (highlight != null) {
            Hexagon<HexagonTile> hex = state.cube2hex(highlight.getLocation());

            g.setColor(Color.ORANGE);
            g.drawOval((int) hex.getCenterX() - 25, (int) hex.getCenterY() - 25, 50, 50);

        }

    }

    protected void paintEntities(Graphics2D g, MapDef map, GameState state) {
        GameSettings settings = model.getSettings();

        for (EntityDef entity : map.getEntityDefs()) {
            Hexagon<HexagonTile> hex = state.cube2hex(entity.getLocation());

            EntityType type = settings.getEntityType(entity.getType());

            SpriteDef spriteDef = type.getSprite();
            if (spriteDef != null) {
                BufferedImage image = sprites.getImage(spriteDef.getImage());
                if (image != null) {
                    int halfWidth = (int) (image.getWidth() * spriteDef.getScale()) / 2;
                    int halfHeight = (int) (image.getHeight() * spriteDef.getScale()) / 2;

                    g.drawImage(
                            image,
                            (int) hex.getCenterX() - halfWidth + spriteDef.getOffset().x,
                            (int) hex.getCenterY() - halfHeight + spriteDef.getOffset().y,
                            (int) (image.getWidth() * spriteDef.getScale()),
                            (int) (image.getHeight() * spriteDef.getScale()),
                            null
                    );

                }
            } else {
                g.setColor(Color.WHITE);
                g.fillOval((int) hex.getCenterX() - 10, (int) hex.getCenterY() - 10, 20, 20);
            }

        }
    }

    protected void paintResources(Graphics2D g2, MapDef map, GameState state) {
        GameSettings settings = model.getSettings();

        for (ResourceDef resource : map.getResourceDefs()) {

            Hexagon<HexagonTile> hex = state.cube2hex(resource.getLocation());
            Shape s = hex2shape.apply(hex);

            ResourceType rt = settings.getResourceType(resource.getName());

            BufferedImage test = sprites.getImage(rt.getImage());
            if (test != null) {

                //no idea what an earth the hexagons bounds are - but this is what we need.
                Rectangle bounds = s.getBounds();

                g2.drawImage(test, (int) bounds.getX(), (int) bounds.getY(), (int) bounds.width + 1, (int) bounds.height + 1, null);
            } else {
                g2.setColor(rt.getColor());
                g2.fillOval((int) hex.getCenterX() - 10, (int) hex.getCenterY() - 10, 20, 20);
            }

            g2.setColor(Color.DARK_GRAY);
            g2.draw(s);

        }
    }


}
